/*
 * Preorder.js
 *
 *  Copyright (C) 2011 Stefan Bolus, University of Kiel, Germany
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


/*
 * Ternary logic disjunction.
 * 
 *    x | y | x or y
 *    --------------
 *    ? | 1 |   1
 *    ? | 0 |   0
 *    ? | ? |   ?
 *    
 * If x,y != ?, then like Boolean disjunction; commutative.
 */
function ternary_or (x,y)
{
	if (x === null && y === null) return null;
	else return x || y;
}



/*!
 * Implements a \e preorder relation. A relation R is called preorder if it is
 * reflexive and transitive.
 * 
 * \param n Number of players.
 * \param is_greater_func Function (i,j) |-> {true,false} which returns true if i is
 * 		greater than j and false otherwise.
 * \param are_equal_func Function (i,j) |-> {true,false} which returns true if i is
 * 		as equal as j and false otherwise.
 * \param pre_are_equal_func Has to be symmetric.
 */
function Preorder (n, is_greater_func, are_equal_func, pre_are_equal_func)
{
	this.n = n;
	this.is_greater_func = is_greater_func;
	this.are_equal_func = are_equal_func;
	
	this.is_total = null;
	this.not_total_witnesses = null; /* Only valid if is_total is false. If so,
									  * contains a pair of incomparable players. */
	this.order = []; // Only valid is is_total is true.
	
	/*! Equivalence classes discovered so far. If two players belong to 
	 * different sets only this.I can rule out equivalence. */
	this.equiv_classes = new UnionFind (this.n);
	
	/*! Indicates if have_equiv_classes are the equiv_classes or if they are
	 * just on the way. */
	this.have_equiv_classes = false;
	
	this.I = []; // equivalence relation.
	this.G = []; /* strict part; irreflexive, asymmetric and transitive,
				  * poset on equiv. classes. */
	
	var i,j;
	
	/* At the beginning we know nothing. */
	for (i = 0 ; i < this.n ; ++i) {
		this.I[i] = [];
		this.G[i] = [];
		for (j = 0 ; j < n ; ++j) {
			this.I[i][j] = null; // MAYBE
			this.G[i][j] = null; 
		}
		
		/* Strict part is irreflexive; I is reflexive. */
		this.G[i][i] = false;
		this.I[i][i] = true;
	}
	
	/* Now we can join together some equivalence classes. */
	if (pre_are_equal_func) {
		for (i = 0 ; i < this.n ; ++i) {
			for (j = 0 ; j < this.n ; ++j) {
				/* It is not necessary to add any information for this.I because
				 * that data will not be used again later. */
				if (i !== j && pre_are_equal_func(i,j))
					this.equiv_classes.union (i,j);
			}
		}
	}
	
	/**************************************************************************
	 *                             Private Operations                         *
	 *************************************************************************/
	
	this._mergeCol = function (mat,j1,j2) {
		if (j1 > j2)
		for (var i = 0 ; i < this.n ; i ++)
			mat[i][j1] = mat[i][j2] = ternary_or(mat[i][j1], mat[i][j2]);
	};
	
	this._mergeRow = function (mat,i1,i2) {
		for (var j = 0 ; j < this.n ; j ++)
			mat[i1][j] = mat[i2][j] = ternary_or(mat[i1][j], mat[i2][j]);
	};
	
	/*! 
	 * i,j are assumed to be the representatives of their corresponding
	 * equivalence classes.
	 */
	this._areEqual = function (i,j) {
		if (i === j) return true;
		else return this.I[i][j];
	};
	
	/*!
	 * Returns true exactly if both players are equally desirable. 
	 */
	this._areEqualImpl = function (i,j) { return this.are_equal_func (i,j); };
	
	/*! 
	 * i,j are assumed to be the representatives of their corresponding
	 * equivalence classes. Returns nothing.
	 */
	this._setEqual = function (i,j,yesno) {
		if (i === j) return;
		else {
			if (yesno) {
				this.equiv_classes.union (i,j);
				
				this.I[i][j] = this.I[j][i] = true;

				/* Merge together all information we have. One of them is the
				 * new representative of the class. */
				this._mergeCol (this.I, i,j);
				this._mergeRow (this.I, i,j);
				
				/* For the transitive part, it is sufficient to use the
				 * representatives of the equivalence classes we have uncovered
				 * so far. */
				this._transitiveStep (i,j);
				this._transitiveStep (j,i);
			}
			else {
				this.I[i][j] = this.I[j][i] = false;
			}
		}
	};
	
	this.areEqual = function (i,j) {
		var r = this.equiv_classes.find(i);
		var s = this.equiv_classes.find(j);
		

		if (this._areEqual(r,s) === null) {
			this._setEqual(r,s,this._areEqualImpl(r,s));
		}
		return this._areEqual(r,s);

	};
	
	/* ----------------------------------------------------- Strict part --- */
	
	/*! 
	 * i,j are assumed to be the representatives of their corresponding
	 * equivalence classes. Returns if i is greater than j.
	 */
	this._isGreater = function (i,j) {
		if (i === j) return false;
		else return this.G[i][j];
	};
	
	/*!
	 * Returns true if and only if player i is more desirable than player j.
	 */
	this._isGreaterImpl = function (i,j) { return this.is_greater_func(i,j); };
	
	
	/*!
	 * for x,y in Reps. do
	 *    if (x > i && j > y)
	 *       set x > y
	 */
	this._transitiveStep = function (i,j) {
		var reps = this.equiv_classes.getReps();
		
		for (var x in reps) {
			for (var y in reps) {
				var x_i = this.G[reps[x]][i], j_y = this.G[j][reps[y]];
				
				if (x_i) this.G[reps[x]][j] = true; // x > i => x > j
				if (j_y) this.G[i][reps[y]] = true; // j > y => i > y
				if (x_i && j_y) this.G[reps[x]][reps[y]] = true; // x > i && j > y => x > y
			}
		}
	};
	
	
	/*! 
	 * i,j are assumed to be the representatives of their corresponding
	 * equivalence classes. Returns nothing.
	 * 
	 * Assumes i != j
	 */
	this._setGreater = function (i,j,yesno) {
		if (yesno) {
			this.G[i][j] = true;
			this.G[j][i] = false;
			
			// 1. Eq. classes are different! I[i][j] = I[j][i] = false;
			this.I[i][j] = this.I[j][i] = false;
			
			// 2. Transitivity. If x > i and j > y then x > j, i > y 
			//    and x > y.
			this._transitiveStep (i,j);
		}
		else {
			this.G[i][j] = false;
		}
	};
	
	
	this.isGreater = function (i,j) {
		var r = this.equiv_classes.find(i);
		var s = this.equiv_classes.find(j);
		
		if (this._isGreater(r,s) === null) {
			this._setGreater(r,s,this._isGreaterImpl(r,s));
		}
		return this._isGreater(r,s);

	};
	
	
	/**************************************************************************
	 *                                 Operations                             *
	 *************************************************************************/
	
	
	
	/*!
	 * If the game is total, the order of the players with respect to the
	 * desirability relation is returned, that is, i comes in front of j
	 * exactly if i is at least as desirable as j. If both players are equally
	 * desirable, the player with the lower index comes first.
	 * 
	 * \throw Error Throws an error if the game is not total.
	 */
	this.getNonIncrOrder = function () {
		if ( !this.isTotal()) throw "The associated game is not total.";
		else return this.order;
	};

	
	/*!
	 * Returns true if and only if the relation is total. With respect to
	 * simple games, this is equivalent to be swap robust and to be complete.
	 * 
	 * \see isDirected
	 */
	this.isTotal = function () {
		if (this.is_total === null) {
			var i;
			var order = [];
			for (i = 0 ; i < this.n ; ++i)
				order[i] = i;
			var thisObject = this;

			/* With respect to ECMA-265 (5th Ed.), Sec. 15.4.4.11, Array.sort
			 * is not _stable_. But we need a stable sorting algorithm for the
			 * non-increasing order of the players. */
			order = mergeSort(order, function (i,j) {
				return thisObject.isGreater.call (thisObject, i,j);
			});
			
			this.is_total = true;
			for (i = 1 ; i < this.n ; ++i) {
				if ( !this.isGreaterEqual(order[i-1], order[i])) {
					//verbose (" !("+(i-1)+" >=_I "+i+")\n");
					this.is_total = false;
					this.not_total_witnesses = [order[i-1],order[i]];
					break;
				}
			}
			
			this.order = order;
		}
		return this.is_total;
	};
	
	
	/*!
	 * If the preorder is not total, returns a pair [i,j] of incomparable
	 * players. If the relation is total returns null.
	 */
	this.getNotTotalWitnesses = function () {
		if (this.isTotal()) return null;
		else return this.not_total_witnesses;
	};
	
	
	/*!
	 * Returns true exactly if player i is at least as desirable as player j.
	 */
	this.isGreaterEqual = function (i, j) {
		return this.isGreater(i,j) || this.areEqual(i,j);
	};
	
	
	/*!
	 * Returns the classes of symmetric (equally desirable) players. The
	 * classes are ordered by non-increasing desirability.
	 */
	this.getEquivClasses = function () {
		if ( !this.have_equiv_classes) {
			this.have_equiv_classes = true;
			
			var i,j;
			
			if (this.isTotal()) {
				/* If the preorder is total the equivalence classes can be
				 * identified with O(n) additional tests. */
				for (i = 0 ; i < this.n-1 ; ++i) {
					if ( !this.isGreater (this.order[i], this.order[i+1]))
						this._setEqual (this.order[i], this.order[i+1]);
				}
			}
			else /* not total */ {
				var reps = this.equiv_classes.getReps();
				var covered = [];
				
				for (i in reps) covered[i] = false;
				
				for (i in reps) {
					if ( !covered[i]) {
						covered[i] = true;
						
						for (j in reps)
							if (this.areEqual(reps[i],reps[j]))
								covered[j] = true;
					}
				}
			}
		}
		return this.equiv_classes;
	};
	
	
	this.direct_succs = []; // representative -> {direct successors}  
	
	/*!
	 * Returns the direct successors of the given element w.r.t. to >, that is,
	 * j is a direct successor of j if i>j and there is no k such that i>k>j.
	 * The direct successors are the same for each element in a equivalence
	 * class.
	 * 
	 * \note Computes the equivalence classes if necessary.
	 */
	this.getDirectSuccessorsOf = function (h) {
		this.getEquivClasses();
		var r = this.equiv_classes.find(h);
		if ( !this.direct_succs[r]) {
			/* Sort the equivalence classes by non-increasing desirability. */
			var reps = this.equiv_classes.getReps();
			var order = reps.slice(0); // copy
			order.sort (function(x,y){return x-y;}); // ascending
			var thisObject = this;
			order = mergeSort (order, function(x,y){ return thisObject.isGreater.call(thisObject,x,y); }); 
			// order contains the order representatives.
			
			var i,j,k;
			var m /*eq. class count*/ = order.length; 
			
			/* Initialize the sets of successors for each equiv. class. */
			for (i in reps)
				this.direct_succs[reps[i]] = [];
			
			if (this.isTotal()) {
				for (i = 0 ; i < m-1 ; ++i)
					this.direct_succs[order[i]] = this.getEquivClassOf(order[i+1]);
			}
			else /* not total */ {
				for (i = 0 ; i < m-1 ; ++i) {
					for (j = i+1 ; j < m; ++j) {
						/* Is j a successor of i? */
						if (this.isGreater(order[i], order[j])) {
							var is_direct = true;
							/* Is j a DIRECT successor of i? */
							for (k = i+1 ; k < j && is_direct ; ++k)
								if (this.isGreater(order[i],order[k]) && this.isGreater(order[k],order[j]))
									is_direct = false;
							
							if (is_direct)
								this.direct_succs[order[i]] = this.direct_succs[order[i]].concat(this.getEquivClassOf(order[j]));
						}
					}
				}
			}
		}
		return this.direct_succs[r];
	};
	
	
	this.direct_succ_reps = []; // representative -> {direct successors}  
	
	 // h don't have to be a rep.
	this.getDirectSuccessingRepsOf = function (h) {
		this.getEquivClasses();
		var r = this.equiv_classes.find(h);
		
		if ( !this.direct_succ_reps[r]) {
			/* Sort the equivalence classes by non-increasing desirability. */
			var reps = this.equiv_classes.getReps();
			var order = reps.slice(0); // copy
			order.sort (function(x,y){return x-y;}); // ascending
			var thisObject = this;
			order = mergeSort (order, function(x,y){ return thisObject.isGreater.call(thisObject,x,y); }); 
			// order contains the order representatives.
			
			var i,j,k;
			var m /*eq. class count*/ = order.length; 
			
			/* Initialize the sets of successors for each equiv. class. */
			for (i in reps)
				this.direct_succ_reps[reps[i]] = [];
			
			if (this.isTotal()) {
				for (i = 0 ; i < m-1 ; ++i)
					this.direct_succ_reps[order[i]] = [order[i+1]];
			}
			else /* not total */ {
				for (i = 0 ; i < m-1 ; ++i) {
					for (j = i+1 ; j < m; ++j) {
						/* Is j a successor of i? */
						if (this.isGreater(order[i], order[j])) {
							var is_direct = true;
							/* Is j a DIRECT successor of i? */
							for (k = i+1 ; k < j && is_direct ; ++k)
								if (this.isGreater(order[i],order[k]) && this.isGreater(order[k],order[j]))
									is_direct = false;
							
							if (is_direct)
								this.direct_succ_reps[order[i]].push(order[j]);
						}
					}
				}
			}
		}
		return this.direct_succ_reps[r];
	};
	
	
	this.reps_to_eq_class = null;
	
	/*!
	 * Returns the set of elements equivalent to the given element.
	 */
	this.getEquivClassOf = function (i) {
		if (null === this.reps_to_eq_class) {
			this.getEquivClasses(); /* Compute the equivalence classes if 
									 * necessary. */
			this.reps_to_eq_class = this.equiv_classes.getSetsOfReps();			
		}
		return this.reps_to_eq_class[this.equiv_classes.find(i)]; 
	};
	
	
	/*!
	 * Should not be used. Dumps a lot of internal information which is useless
	 * for the end user. Does not apply any tests but instead dumps the info
	 * gathered so far.
	 */
	this.toString = function () {
		var i,j;
		
		var s = "___AtLeastAsDesirable:___\n"
			+"             n : " + this.n + "\n"
			+"      is_total : " + this.is_total + "\n"
			+"         order : " + this.order + "\n"
			+" equiv_classes : " + this.equiv_classes + " (fully dicovered? "+this.have_equiv_classes+")" + "\n"
			+"  direct_succs : \n";
		
		for (i in this.direct_succs) {
			s += "             {"+ this.getEquivClassOf(i).join(",") +"} > {"+this.direct_succs[i].join(",")+"}\n";
		}
		
		s += "             I : (equally desirable)\n";
		for (i = 0 ; i < this.n ; ++i) {
			s += "               ";
			for (j = 0 ; j < this.n ; ++j)
				s += this.I[i][j] ? "=" : ((this.I[i][j] === null) ? "?" : ".");
			s += "\n";
		}
		s += "             G : (more desirable)\n";
		for (i = 0 ; i < this.n ; ++i) {
			s += "               ";
			for (j = 0 ; j < this.n ; ++j)
				s += this.G[i][j] ? ">" : ((this.G[i][j] === null) ? "?" : ".");
			s += "\n";
		}
		return s;
	};
}


